(function(){//Router Configuration
Router.configure({
  layoutTemplate: 'layout',
  loadingTemplate: 'loading',
  notFoundTemplate: 'notFound',
  waitOn: function() {
    return Meteor.subscribe('devices');
  }
});

//Filter to allow only logged in users
var requireLogin = function() {
  if (! Meteor.user()) {
    if (Meteor.loggingIn()) {
      this.render(this.loadingTemplate);
    } else {
      this.render('accessDenied');
    }
  } else {
    this.next();
  }
};
//Router hooks
if (Meteor.isClient) {
  Router.onBeforeAction(requireLogin, {except: ['accessDenied']});
  Router.onBeforeAction('dataNotFound', {only: 'devicePage'});
  Router.onBeforeAction('loading');
}

//Application routes
Router.route('/device', {name: 'devicesList'});
Router.route('/device/create', {name: 'deviceCreate'});
Router.route('/device/:_id/report', {
  name: 'deviceReport',
  waitOn: function() {
    return Meteor.subscribe('reportsForChart', function () {
      console.log(arguments);
      Reports.find().observe({
        changed: function (id, fields) {
          console.log(fields);
          Meteor.call('updateSeriesData');
        },
        removed: function (id) {
          Meteor.call('updateSeriesData');
        },
        added: function (id) {
          Meteor.call('updateSeriesData');
        }
      });
    });
  },
  data: function () {
    Session.set('currentDevice', this.params._id);
    return Devices.findOne(this.params._id);
  }
});
Router.route('/device/:_id/sensor', {
  name: 'sensorsList',
  data: function () {
    Session.set('currentDevice', this.params._id);
    return Devices.findOne(this.params._id);
  }
});
Router.route('/device/:_id/sensor/create', {
  name: 'sensorCreate',
  data: function () {
    Session.set('currentDevice', this.params._id);
    return Devices.findOne(this.params._id);
  }
});

//Server Routes to record data from devices.
if (Meteor.isServer) {
  Router.route('/api/device/:deviceid/sensor/:slug/report', {where: 'server'}).get(function () {
    Meteor.call('recordData', this.params.deviceid, this.params.slug, this.params.query.value, function (error, result) {
      if (error) {
        console.log(error);
      }
    });
    this.response.end('ok');
  });
}

})();
